@@ -17,36 +17,32 @@ from bakplane.bakplane_pb2 import (
 
 date_constraint_t = typing.Union[
     typing.Tuple[datetime.datetime, datetime.datetime],
-    typing.List[typing.Tuple[datetime.datetime, datetime.datetime, str]],
+    typing.Tuple[datetime.datetime, datetime.datetime, str],
     datetime.datetime,
 ]
 
+
+DT_COLUMN_NAME = 'dt'
+DEFAULT_SERIES = ['B', 'D', 'W', 'M', 'SM', 'BM', 'MS', 'SMS', 'BMS', 'Q', 'BQ', 'QS',  'Y', 'BY', 'YS', 'BYS']
 INFINITY_PROXY_DATE = datetime.datetime(year=3000, month=12, day=31)
 
 
 def flatten_date_constraints(c: date_constraint_t) -> typing.List[datetime.datetime]:
-    if isinstance(c, tuple):
-        return [c[0], c[1]]
-    elif isinstance(c, datetime.datetime):
+    if isinstance(c, datetime.datetime):
         return [c, INFINITY_PROXY_DATE]
-    elif isinstance(c, list):
-        if len(c) <= 0:
-            raise ValueError('Date constraints must contain at least one element.')
+    elif isinstance(c, tuple):
+        if len(c) == 2:
+            return [c[0], c[1]]
+        elif len(c) == 3:
+            results = set()
 
-        results = set()
-
-        for entry in c:
-            if len(entry) != 3:
-                raise ValueError('Date constraint tuple must contain exactly 3 parts: (start, end, frequency).')
-
-            start, end, freq = entry
+            start, end, freq = c
             ranges = pd.bdate_range(start=start, end=end, freq=freq)
             for r in ranges:
                 results.add(r.to_pydatetime())
-
-        return list(results)
-    else:
-        raise RuntimeError('Date constraints is not valid.')
+            return list(results)
+        else:
+            raise RuntimeError(f'Invalid dating definition: `{c}` is not valid.')
 
 
 class WriteMode(Enum):
@@ -94,6 +90,9 @@ class ReadRequestBuildingBlocks:
     effective_start_dt: datetime = None
     effective_end_dt: datetime = None
 
+    valid_start_dt: datetime = None
+    valid_end_dt: datetime = None
+
     knowledge_dt: datetime.datetime = None
     query: typing.Any = None
     columns: typing.List[Column] = None
@@ -110,6 +109,14 @@ class DataStoreHelper(ABC):
     ) -> Table:
         pass
 
+    @abstractmethod
+    def get_or_create_series_temporary_table(self, frequency: typing.Tuple[datetime.datetime, datetime.datetime, str]) -> Table:
+        pass
+
+    @abstractmethod
+    def initialize_series(self):
+        pass
+
 
 class Modifier:
     @abstractmethod
@@ -127,17 +134,8 @@ class ReadRequest:
 
     pointers: typing.Dict[str, str]
 
-    effective_dating: typing.Union[
-        typing.Tuple[datetime.datetime, datetime.datetime],
-        typing.List[typing.Tuple[datetime.datetime, datetime.datetime, str]],
-        datetime.datetime
-    ] = None
-
-    valid_dating: typing.Union[
-        typing.Tuple[datetime.datetime, datetime.datetime],
-        typing.List[typing.Tuple[datetime.datetime, datetime.datetime, str]],
-        datetime.datetime
-    ] = None
+    effective_dating: date_constraint_t = None
+    valid_dating: date_constraint_t = None
 
     columns: typing.List[str] = None
     knowledge_dt: datetime.datetime = None
