@@ -1,8 +1,6 @@
 import random
 import typing
 import urllib.parse
-import tempfile
-
 from datetime import datetime
 from timeit import default_timer as timer
 
@@ -21,7 +19,10 @@ from bakplane.extensions.base import (
     ReadRequest,
     ReadRequestBuildingBlocks,
     DataStoreHelper,
-    INFINITY_PROXY_DATE
+    INFINITY_PROXY_DATE,
+    DT_COLUMN_NAME,
+    flatten_date_constraints,
+    DEFAULT_SERIES
 )
 from bakplane.extensions.spark.utils import (
     to_spark_save_mode,
@@ -41,8 +42,10 @@ class JDBCStoreHelper(DataStoreHelper):
             database: str,
             schema_name: str,
             driver: str,
-            table_prefixes: typing.List[str] = None
+            table_prefixes: typing.List[str] = None,
+            series_schema: str = 'series',
     ):
+
         self.metadata = MetaData()
         self.engine = create_engine(
             f"{driver}://{user}:{password}@{host}:{port}/{database}",
@@ -52,6 +55,57 @@ class JDBCStoreHelper(DataStoreHelper):
         )
         self.schema_name = schema_name
         self.table_prefixes = table_prefixes
+        self.series_schema = series_schema
+        self.frequency_to_tbl = {}
+
+        self.initialize_series()
+
+    def get_or_create_series_temporary_table(self, series: typing.Tuple[datetime, datetime, str]) -> Table:
+        freq = series[2].strip().lower()
+        t_name = f't_{freq}_{str(int(datetime.utcnow().timestamp()))}_{str(random.randint(1, 100000))}'
+
+        if freq in self.frequency_to_tbl:
+            return self.frequency_to_tbl[freq].alias(t_name)
+
+        rows = [{DT_COLUMN_NAME: x} for x in flatten_date_constraints(series)]
+        if len(rows) <= 0:
+            raise RuntimeError('Date constraints must span at least one day.')
+
+        tbl = self.create_temporary_table(
+            prefix='t_series',
+            columns=[Column(
+                DT_COLUMN_NAME,
+                DateTime,
+                nullable=False
+            )],
+            rows=rows
+        )
+
+        self.frequency_to_tbl[freq] = tbl
+        return tbl.alias(t_name)
+
+    def initialize_series(self):
+        for s in DEFAULT_SERIES:
+
+            freq = s.lower().strip()
+            tbl_name = f's_{freq}'
+
+            columns = [Column(
+                DT_COLUMN_NAME,
+                DateTime,
+                nullable=False,
+                primary_key=True
+            )]
+
+            tbl = Table(
+                tbl_name,
+                self.metadata,
+                *columns,
+                schema=self.series_schema
+            )
+
+            if self.engine.dialect.has_table(self.engine, table_name=tbl_name, schema=self.series_schema):
+                self.frequency_to_tbl[freq] = tbl
 
     def create_temporary_table(
             self,
@@ -86,9 +140,13 @@ def __build_read_all_query(
     return f'select {", ".join(r.columns)} from {res.context["schema_name"]}.{res.context["table_name"]}'
 
 
-def __add_effective_dating_constraint(blocks: ReadRequestBuildingBlocks, res: ResolveResourceIntentResponse):
+def __add_effective_dating_constraint(blocks: ReadRequestBuildingBlocks, res: ResolveResourceIntentResponse,
+                                      r: ReadRequest):
     a = blocks.asset_table
 
+    if r.effective_dating is None:
+        return
+
     if blocks.effective_dating_table is not None:
         blocks.constraints.append(
             blocks.effective_dating_table.c.dt >= a.c.effective_start_dt
@@ -97,6 +155,15 @@ def __add_effective_dating_constraint(blocks: ReadRequestBuildingBlocks, res: Re
         blocks.constraints.append(
             blocks.effective_dating_table.c.dt < a.c.effective_end_dt
         )
+
+        blocks.constraints.append(
+            blocks.effective_dating_table.c.dt >= datetime_to_date_string(blocks.effective_start_dt)
+        )
+
+        blocks.constraints.append(
+            blocks.effective_dating_table.c.dt < datetime_to_date_string(blocks.effective_end_dt)
+        )
+
     else:
         blocks.constraints.append(
             a.c.effective_start_dt
@@ -124,40 +191,49 @@ def __add_valid_dating_constraint(blocks: ReadRequestBuildingBlocks, res: Resolv
             blocks.valid_dating_table.c.dt < a.c.valid_end_dt
         )
 
+        blocks.constraints.append(
+            blocks.valid_dating_table.c.dt >= datetime_to_date_string(blocks.valid_start_dt)
+        )
 
-def __populate_building_blocks_effective_dating(helper: DataStoreHelper, blocks: ReadRequestBuildingBlocks, r: ReadRequest):
-    if isinstance(r.effective_dating, list):
-        blocks.effective_dating_table = helper.create_temporary_table('t_dating', [Column('dt', DateTime, nullable=False)], [{'dt': x} for x in r.get_resolved_effective_dating()])
-    elif isinstance(r.effective_dating, tuple):
-        start, end = r.effective_dating
-
-        blocks.effective_start_dt = start
-        blocks.effective_end_dt = end
-    elif isinstance(r.effective_dating, datetime):
-        blocks.effective_start_dt = r.effective_dating
-        blocks.effective_end_dt = INFINITY_PROXY_DATE
-    else:
-        raise ValueError('Effective dating has unknown type.')
+        blocks.constraints.append(
+            blocks.valid_dating_table.c.dt < datetime_to_date_string(blocks.valid_end_dt)
+        )
 
 
-def __populate_building_blocks_valid_dating(helper: DataStoreHelper, blocks: ReadRequestBuildingBlocks, r: ReadRequest):
+def __populate_building_blocks_dating(request_field: str, start_field: str, end_field: str, dating_tbl_field: str,
+                                      helper: DataStoreHelper, blocks: ReadRequestBuildingBlocks, r: ReadRequest):
+    if not hasattr(r, request_field):
+        raise RuntimeError('Invalid field name.')
 
-    if r.valid_dating is None:
+    dating = getattr(r, request_field)
+
+    if dating is None:
         return
 
-    if isinstance(r.valid_dating, list):
-        blocks.valid_dating_table = helper.create_temporary_table('t_dating', [Column('dt', DateTime, nullable=False)], [{'dt': x} for x in r.get_resolved_valid_dating()])
-    elif isinstance(r.valid_dating, tuple):
-        start, end = r.valid_dating
+    if isinstance(dating, tuple):
+        if len(dating) == 2:
+            start, end = dating
 
-        blocks.valid_start_dt = start
-        blocks.valid_end_dt = end
+            setattr(blocks, start_field, start)
+            setattr(blocks, end_field, end)
 
-    elif isinstance(r.valid_dating, datetime):
-        blocks.valid_start_dt = r.valid_dating
-        blocks.valid_end_dt = INFINITY_PROXY_DATE
+            blocks.effective_start_dt = start
+            blocks.effective_end_dt = end
+
+        elif len(dating) == 3:
+
+            setattr(blocks, dating_tbl_field, helper.get_or_create_series_temporary_table(dating))
+            setattr(blocks, start_field, dating[0])
+            setattr(blocks, end_field, dating[1])
+
+        else:
+            raise RuntimeError('Invalid dating constraint.')
+
+    elif isinstance(dating, datetime):
+        setattr(blocks, start_field, dating)
+        setattr(blocks, end_field, INFINITY_PROXY_DATE)
     else:
-        raise ValueError('Valid dating has unknown type.')
+        raise ValueError('Effective dating has unknown type.')
 
 
 def __build_read_query(
@@ -174,6 +250,9 @@ def __build_read_query(
         and_,
     )
 
+    if (r.columns is None or len(r.columns) <= 0) and r.query_builder is None:
+        raise RuntimeError("You must provide a set of columns.")
+
     metadata: MetaData = MetaData()
 
     src_tbl: Table = Table(
@@ -219,11 +298,25 @@ def __build_read_query(
         ])],
     )
 
-    __populate_building_blocks_effective_dating(store_helper, blocks, r)
-    __populate_building_blocks_valid_dating(store_helper, blocks, r)
+    __populate_building_blocks_dating(
+        'effective_dating',
+        'effective_start_dt',
+        'effective_end_dt',
+        'effective_dating_table',
+        store_helper,
+        blocks,
+        r
+    )
 
-    if (r.columns is None or len(r.columns) <= 0) and r.query_builder is None:
-        raise RuntimeError("You must provide a set of columns.")
+    __populate_building_blocks_dating(
+        'valid_dating',
+        'valid_start_dt',
+        'valid_end_dt',
+        'valid_dating_table',
+        store_helper,
+        blocks,
+        r
+    )
 
     if r.query_builder:
         res = r.query_builder(blocks)
@@ -233,19 +326,27 @@ def __build_read_query(
         if r.additional_constraints is not None:
             blocks.constraints.append(r.additional_constraints(blocks))
 
+        if r.effective_dating is not None:
+            blocks.constraints.append(
+                a.c.effective_start_dt <= blocks.mapping_table.c.effective_end_dt
+            )
+
+            blocks.constraints.append(
+                a.c.effective_end_dt >= blocks.mapping_table.c.effective_start_dt
+            )
+        else:
+            blocks.constraints.append(
+                a.c.valid_start_dt <= blocks.mapping_table.c.effective_end_dt
+            )
+
+            blocks.constraints.append(
+                a.c.valid_end_dt >= blocks.mapping_table.c.effective_start_dt
+            )
+
         __add_valid_dating_constraint(blocks, res)
 
         if r.effective_dating_hint is None:
-
-            blocks.constraints.append(
-                a.c.effective_start_dt <= m.c.effective_end_dt
-            )
-
-            blocks.constraints.append(
-                a.c.effective_end_dt >= m.c.effective_start_dt
-            )
-
-            __add_effective_dating_constraint(blocks, res)
+            __add_effective_dating_constraint(blocks, res, r)
 
         else:
             if isinstance(r.effective_dating_hint, tuple):
